# odoo-api
This project provides the state51-Odoo library to be able to develop platform applications.

# Synopsis
```
    use state51::Odoo;

    my $odoo = state51::Odoo->new();
    my $uid = $odoo->odoo_uid;

    my $ids = $odoo->execute(
            'account.move',
            'search',
            [
                [ 'ref', '>=',   $self->invoice_prefix . $invoice_number ],
                [ 'ref', 'like', $prefix . "%" ]
            ]
    );

    my $results = $odoo->execute($model, 'read', $ids, [qw/ref/]);
```

# Usage examples
```
    # get all invoices id
    my $ids = $odoo->execute(
        'account.move',
        'search',
        []
    );

    # get information about invoices
    my $ids = $odoo->execute(
        'account.move',
        'read',
        $ids,
        [qw/ref/] #list of fields
    );

    # get information about invoices without Frontier::RPC2::Boolean objects
    my $ids = $odoo->execute(
        'account.move',
        'read',
        $ids,
        [qw/ref/], #list of fields
        {replace_boolean => 1}
    );

    # get all partners id
    my $partners_id = $odoo->execute(
        'res.partner',
        'search',
        []
    );

    # get information about partners
    my $partners = $odoo->execute(
        'res.partner',
        'read',
        [$partners_id->[0]],
        [qw/contract_ids invoice_ids name email/] #list of fields
    );

    # download invoice report in file
    my $report_filename = 'report_invoce_file';
    my $response = $self->{odoo}->download_invoice_report($report_filename, $invoice_id);

    # get listing record fields
    my $fields_info = $odoo->execute(
        'res.partner',
        'fields_get',
        [qw/property_account_position_id/], # an empty list will return all fields in the model
        [qw/string help type/], # an empty list will return all the attributes of the field
    );
    print Dumper($fields_info);
```


# Enviroment
The odoo library works with odoo via an external api. You need to log in and connect to the database. The desired configuration can be set when creating an object:
```
    my $odoo = state51::Odoo->new(
        user     => $user,
        password => $passsword,
        db_name  => $db_name,
        base_url => $base_url,
    );
```
Or you can also set environment variables:
```
export ODOO_USERNAME=<user name>
export OODOO_PASSWORD=<password>
export OODOO_DB=<db name>
export OODOO_URL=<base url>
```

# Testing
For testing, we use a special testing odoo.
```
        db_name  => 'odoo_15_dev',
        base_url => 'http://odoo-dev.int.openimp.com',
```

# Documentation odoo api
External API odoo: https://www.odoo.com/documentation/15.0/developer/api/external_api.html
