package state51::Odoo::Test;

use strict;
use warnings;
use base qw(Test::Class);

use Test::Most;
use state51::Odoo;
use Data::Dumper;
use state51::Odoo::TestCommon;

sub _startup : Test(startup => 2) {
    my $self = shift;
    $self->check_that_we_use_required_env_variables();
}

sub _setup : Test(setup) {
    my $self = shift;

    # Force the usage of the *_DEV environment variables
    $self->{odoo} = state51::Odoo->new(
        user     => $ENV{ODOO_USERNAME},
        password => $ENV{ODOO_PASSWORD},
        db_name  => $ENV{ODOO_DB_DEV},
        base_url => $ENV{ODOO_URL_DEV},
    );
    $self->{model} = 'account.move';
}

sub test_odoo_uid : Test(1) {
    my $self = shift;

    my $odoo = $self->{odoo};

    my $uid = $odoo->odoo_uid;
    ok($uid > 0, 'uid');
}

sub test_execute : Test(4) {
    my $self = shift;

    my $odoo = $self->{odoo};

    my $initial_ids = $odoo->execute(
        $self->{model},
        'search',
        []
    );

    my $new_entry = {
        partner_id => 1,
        state => 'draft',
    };

    my $new_id = $odoo->execute(
        $self->{model},
        'create',
        $new_entry
    );

    my $ids_after_create = $odoo->execute(
        $self->{model},
        'search',
        []
    );

    is_deeply([sort @$ids_after_create], [sort @$initial_ids, $new_id], 'execute - create');

    my $new_invoice = $odoo->execute(
        $self->{model},
        'read',
        [$new_id],
        [qw/has_message campaign_id edi_error_count/],
    );
    delete $new_invoice->[0]{id};

    is_deeply(
        $new_invoice,
        [{
            campaign_id => bless( do{\(my $o = '0')}, 'Frontier::RPC2::Boolean' ),
            edi_error_count => '0',
            has_message => bless( do{\(my $o = '1')}, 'Frontier::RPC2::Boolean' )
        }],
        'execute - read'
    );

    my $new_invoice_rep_bool = $odoo->execute(
        $self->{model},
        'read',
        [$new_id],
        [qw/has_message campaign_id edi_error_count/],
        {replace_boolean => 1}
    );
    delete $new_invoice_rep_bool->[0]{id};

    is_deeply(
        $new_invoice_rep_bool,
        [{
            campaign_id => undef,
            edi_error_count => '0',
            has_message => '1'
        }],
        'execute - read replace bool'
    );

    $odoo->execute(
        $self->{model},
        'unlink',
        [$new_id]
    );

    my $ids_after_unlink = $odoo->execute(
        $self->{model},
        'search',
        []
    );

    is_deeply($initial_ids, $ids_after_unlink, 'execute - unlink');
}

sub test_www_mechanize : Test(2) {
    my $self = shift;

    my $odoo_invoice_id = $self->create_new_invoice();
    my $report_filename = 'report_invoce_file_for_test';
    $self->delete_file($report_filename);

    my $response = $self->{odoo}->download_invoice_report($report_filename, $odoo_invoice_id);
    
    ok($response, 'We received a response after asking to download the invoice');
    ok(-f $report_filename, 'A file exist');

    $self->delete_file($report_filename);
    $self->delete_invoice($odoo_invoice_id);
}

sub delete_invoice {
    my ($self, $odoo_invoice_id) = @_;

    $self->{odoo}->execute(
        $self->{model},
        'unlink',
        [$odoo_invoice_id]
    );
}

sub create_new_invoice {
    my $self = shift;

    my $new_entry = {
        # That's Pauline test shop, check it's fine to use
        partner_id => 18463,
        state => 'draft',
        move_type => 'out_invoice',
        invoice_date => '2022-09-01',
    };

    return $self->{odoo}->execute(
        $self->{model},
        'create',
        $new_entry
    );
}

sub delete_file {
    my ($report_filename) = @_;
    if (-f $report_filename) {
        `rm $report_filename`;
    }
}

1;
