package state51::Odoo::Test;

use strict;
use warnings;
use base qw(Test::Class);

use Test::Most;
use state51::Odoo;
use state51::Odoo::TestCommon;

sub _startup : Test(startup => 2) {

    my $self = shift;

    $self->check_that_we_use_required_env_variables();

    # XXX No idea why this is required
    no warnings qw(redefine);
    *Frontier::Client::new = sub {
        return bless {}, $_[0];
    };

    *Frontier::Client::call = sub {
        if ($_[1] eq 'execute') {
            if ($_[6] eq 'read' and defined $_[8]) {
                return [
                    {
                        ref => 'foo',
                        bool_false => bless( do{\(my $o = '0')}, 'Frontier::RPC2::Boolean'),
                        bool_true  => bless( do{\(my $o = '1')}, 'Frontier::RPC2::Boolean')
                    }
                ];
            }

            return [qw/3 2 1/];
        }

        # XXX It defeat the whole odoo_uid thing?
        #     But if we remove, it the tests fails...
        if ($_[1] eq 'login') {
            # sam@openimp.com's id
            return 86;
        }
        return;
    };
}

sub _setup : Test(setup) {
    my $self = shift;

    # Force the usage of some *_DEV environment variables, when necessary
    $self->{odoo} = state51::Odoo->new(
        user     => $ENV{ODOO_USERNAME},
        password => $ENV{ODOO_PASSWORD},
        db_name  => $ENV{ODOO_DB_DEV},
        base_url => $ENV{ODOO_URL_DEV},
    );
}

sub test_main : Test(5) {
    my $self = shift;

    my $odoo = $self->{odoo};

    my $env_linking_table = {
        user     => 'ODOO_USERNAME',
        password => 'ODOO_PASSWORD',
        db_name  => 'ODOO_DB_DEV',
        base_url => 'ODOO_URL_DEV',
    };

    for my $key (keys $env_linking_table->%*) {
        my $corresponding = $env_linking_table->{$key};
        is($odoo->{$key}, $ENV{$corresponding}, 'main test - ' . $key);
    }

    my $uid = $odoo->odoo_uid;

    ok($uid > 0, 'main test - We managed to login (retrieve an uid)');
}

sub test_execute : Test(7) {
    my $self = shift;

    my @tests_data = (
        {
            name => 'ordinary',
            data => {
                model    => 'test',
                method   => 'search',
                position =>
                  [ [ 'ref', '>=', 0 ], [ 'ref', 'like', 'test' . '%' ] ],
            },
            expect => [qw/3 2 1/],
        },
        {
            name => 'empty model',
            data => {
                model    => undef,
                method   => 'search',
                position =>
                  [ [ 'ref', '>=', 0 ], [ 'ref', 'like', 'test' . '%' ] ],
            },
            expect => undef,
            warn => '[Odoo::execute] Model or method is empty',
        },
        {
            name => 'empty method',
            data => {
                model    => 'test',
                method   => undef,
                position =>
                  [ [ 'ref', '>=', 0 ], [ 'ref', 'like', 'test' . '%' ] ],
            },
            expect => undef,
            warn => '[Odoo::execute] Model or method is empty',
        },
        {
            name => 'with keyword',
            data => {
                model    => 'test',
                method   => 'read',
                position => [],
                keyword  => ['ref', 'bool'],
            },
            expect => [{
                    ref => 'foo',
                    bool_false => bless( do{\(my $o = '0')}, 'Frontier::RPC2::Boolean'),
                    bool_true => bless( do{\(my $o = '1')}, 'Frontier::RPC2::Boolean'),
                }],
        },
        {
            name => 'with args, bool=undef',
            data => {
                model    => 'test',
                method   => 'read',
                position => [],
                keyword  => ['ref', 'bool'],
                args     => {replace_boolean => 1},
            },
            expect => [{ref => 'foo', bool_false => undef, bool_true => 1}],
        },
    );

    my $odoo = $self->{odoo};
    for my $t (@tests_data) {
        local *state51::Odoo::logger = sub {
            $self;
        };
        local *message = sub {
            if ($t->{warn}) {
                my (undef, %args) = @_;
                is($args{message}[0], $t->{warn}, 'execute - ' . $t->{name});
            }
        };
        my $got = $odoo->execute(
            $t->{data}{model},
            $t->{data}{method},
            $t->{data}{position},
            $t->{data}{keyword} ? $t->{data}{keyword} : (),
            $t->{data}{args} ? $t->{data}{args} : (),
        );

        is_deeply($got, $t->{expect}, 'execute - ' . $t->{name});
    }
}

1;
